/*
 * Functions ported from XFree86 Xorg 7.0 (2004,2006)
 * Code is only for dememonstrating backligt control over ioport.
 *
 * Compile:
 *	gcc -O2 -Wall -o BLSignalVDD BLSignalVDD.c
 *
 * Run:
 *	./BLSignalVDD 0		(Backlight off)
 *	./BLSignalVDD 1		(Backlight on)
 */

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>	/* for libc5 */
#include <sys/io.h>	/* ioperm, inp, outp */
#include <stdlib.h>
#include <errno.h>

/* interrupt control.. */
#define sti() __asm__ __volatile__ ("sti": : :"memory")
#define cli() __asm__ __volatile__ ("cli": : :"memory")
#define save_flags(x) \
__asm__ __volatile__("pushfl ; popl %0":"=g" (x): /* no input */ :"memory")
#define restore_flags(x) \
__asm__ __volatile__("pushl %0 ; popfl": /* no output */ :"g" (x):"memory")

#define USHORT uint16_t
#define UCHAR uint8_t

typedef struct {
    USHORT P3d4;
} VB_DEVICE_INFO;

typedef VB_DEVICE_INFO * PVB_DEVICE_INFO;

void XGINew_SetReg1( int addr , USHORT index , USHORT data )
{
    cli();
    outb(index, addr);
    outb(data, addr+1);
    sti();
}

UCHAR XGINew_GetReg1(int addr, unsigned char index)
{
    unsigned char u8;

    cli();
    outb(index, addr);
    u8 = inb(addr+1);
    sti();

    return u8;
}

/* --------------------------------------------------------------------- */
/* Function : XGINew_SetRegANDOR */
/* Input : */
/* Output : */
/* Description : */
/* --------------------------------------------------------------------- */
void XGINew_SetRegANDOR( USHORT Port , USHORT Index , USHORT DataAND , USHORT DataOR )
{
    USHORT temp ;

    temp = XGINew_GetReg1( Port , Index ) ;		/* XGINew_Part1Port index 02 */
    temp = ( temp & ( DataAND ) ) | DataOR ;
    XGINew_SetReg1( Port , Index , temp ) ;
}

/* --------------------------------------------------------------------- */
/* Function : XGINew_SetRegAND */
/* Input : */
/* Output : */
/* Description : */
/* --------------------------------------------------------------------- */
void XGINew_SetRegAND(USHORT Port,USHORT Index,USHORT DataAND)
{
    USHORT temp ;

    temp = XGINew_GetReg1( Port , Index ) ;	/* XGINew_Part1Port index 02 */
    temp &= DataAND ;
    XGINew_SetReg1( Port , Index , temp ) ;
}

UCHAR XG21GPIODataTransfer(UCHAR ujDate)
{
    UCHAR  ujRet = 0;
    UCHAR  i = 0;

    for (i=0; i<8; i++)
	{
    	ujRet = ujRet << 1;
        ujRet |= (ujDate >> i) & 1;
    }

	return ujRet;
}


/*----------------------------------------------------------------------------*/
/* input                                                                      */
/*      bl[5] : 1;LVDS signal on                                              */
/*      bl[1] : 1;LVDS backlight on                                           */
/*      bl[0] : 1:LVDS VDD on                                                 */
/*      bh: 100000b : clear bit 5, to set bit5                                */
/*          000010b : clear bit 1, to set bit1                                */
/*          000001b : clear bit 0, to set bit0                                */
/*----------------------------------------------------------------------------*/
void XGI_XG21BLSignalVDD(USHORT tempbh,USHORT tempbl, PVB_DEVICE_INFO pVBInfo)
{
    UCHAR CR4A,temp;
    
    CR4A = XGINew_GetReg1( pVBInfo->P3d4 , 0x4A ) ;
    tempbh &= 0x23;
    tempbl &= 0x23;
    XGINew_SetRegAND( pVBInfo->P3d4 , 0x4A , ~tempbh ) ; /* enable GPIO write */

    if (tempbh&0x20)
    {
      temp = (tempbl>>4)&0x02;
      
      XGINew_SetRegANDOR( pVBInfo->P3d4 , 0xB4 , ~0x02 , temp) ; /* CR B4[1] */
    
    }
    
    temp = XGINew_GetReg1( pVBInfo->P3d4 , 0x48 ) ;
    
    temp = XG21GPIODataTransfer(temp);
    temp &= ~tempbh;
    temp |= tempbl;
    XGINew_SetReg1( pVBInfo->P3d4 , 0x48 , temp ) ; 
}

#if 0
void XGI_XG27BLSignalVDD(USHORT tempbh,USHORT tempbl, PVB_DEVICE_INFO pVBInfo)
{
    UCHAR CR4A,temp;
    USHORT tempbh0,tempbl0;

    tempbh0 = tempbh;
    tempbl0 = tempbl;
    tempbh0 &= 0x20;
    tempbl0 &= 0x20;
    tempbh0 >>= 3;
    tempbl0 >>= 3;

    if (tempbh&0x20)
    {
      temp = (tempbl>>4)&0x02;
      
      XGINew_SetRegANDOR( pVBInfo->P3d4 , 0xB4 , ~0x02 , temp) ; /* CR B4[1] */
    
    }
    XGINew_SetRegANDOR( pVBInfo->P3d4 , 0xB4 , ~tempbh0 , tempbl0 ) ;

    CR4A = XGINew_GetReg1( pVBInfo->P3d4 , 0x4A ) ;
    tempbh &= 0x03;
    tempbl &= 0x03;
    tempbh <<= 2;
    tempbl <<= 2;                                       /* GPIOC,GPIOD */
    XGINew_SetRegAND( pVBInfo->P3d4 , 0x4A , ~tempbh ) ; /* enable GPIO write */
    XGINew_SetRegANDOR( pVBInfo->P3d4 , 0x48 , ~tempbh , tempbl ) ;
}
#endif

int main (int argc, char *argv[])
{
    VB_DEVICE_INFO VBInfo;
    PVB_DEVICE_INFO pVBInfo = &VBInfo;

    pVBInfo->P3d4 = 0x3d4;

    printf("XG21BLSignalVDD v1.0\n");
    fflush(stdout);
    sleep(0);

    if (argc == 2) {

	if (geteuid() != 0) {
	    printf("Program need root rights\n");
	    exit(EACCES);
	}

	/* full IO-access under linux */
	iopl(3);

	if (argv[1][0] & 1) {
	    XGI_XG21BLSignalVDD( 0x02 , 0x02, pVBInfo ) ; /* LVDS backlight on */
	} else {
	    XGI_XG21BLSignalVDD( 0x02 , 0x00, pVBInfo ) ; /* LVDS backlight off */
	}
    } else {
	printf("Usage: %s [0|1]\n", argv[0]);
    }

    return 0;
}
